body {
  background-color: white;
  color: black;
  margin-left: 1.5em;
  margin-right: 1.5em;
  margin-top: 1.5em;
  margin-bottom: 1em;  
}

ul.changelog_entries {
  margin-left: 0.7em;
  margin-right: 0.7em;
  padding-left: 0.7em;
  padding-right: 0.7em;
  padding-bottom: 0.7em;
  background: #fefefe;
  border: 1px dashed #88aa88;
}

li.changelog_entry {
  list-style-type: none;
  margin-left: 0px;
  padding-left: 0px;
  margin-top: 0.8em;
  border-top: 1px solid #dddddd;
  background: #f8f8f8;
}

li.changelog_change {
  list-style-type: circle;
  margin-left: 4em;
}

span.changelog_date {
  color: black;
}

span.changelog_author {
  color: #111188;
}

.changelog_revision {
  font-size: 80%;
  color: #881111;
  background: #fff4f4;
}

.changelog_revision a {
  color: inherit;
}

.changelog_files {
  font-size: 80%;
  font-family: monospace;
  color: #116611;
}

.changelog_files:after {
  content: ':';
}

.changelog_message {
  display: block;
  color: #220000;
}

p.changelog_footer {
  margin-top: 1.5em;
  margin-left: 0.7em;
  font-style: italic;
  line-height: 90%;
  color: gray;
  font-family: Helvetica, Arial, sans-serif;
}

p.changelog_footer a {
  text-decoration: none;
  color: inherit;
}
